#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pogo/types.h>
#include <pogo/screen.h>
#include <pogo/font.h>

#include "util.h"

#ifdef _WIN32
#define SEPARATOR '\\'
#include <io.h>

struct dirent
{
	char *d_name;
};

typedef struct
{
	struct dirent de;
	struct _finddata_t fd;
	intptr_t handle;
} DIR;


DIR *opendir(char *name)
{
	char tmp[256];
	DIR *dir = (DIR*)malloc(sizeof(DIR));
	dir->de.d_name = NULL;
	sprintf(tmp, "%s\\*", name);
	//printf("Looking for %s\n", tmp);
	dir->handle = _findfirst(tmp, &dir->fd);
	return dir;
}

struct dirent *readdir(DIR *dir)
{
	intptr_t rc = dir->handle;
	if(dir->de.d_name)
		rc = _findnext(dir->handle, &dir->fd);
	if(rc == -1)
		return NULL;
	dir->de.d_name = dir->fd.name;
	return &dir->de;
}

void closedir(DIR *dir)
{
	_findclose(dir->handle);
	free(dir);
}
#else
#define SEPARATOR '/'
#include <dirent.h>
#endif

FileInfo *ReadDir(char *dirname, int *count, char *ext)
{
	char tmp[128];
	FileInfo flist[128];
	DIR *dir;
	struct dirent *de;
	int i = 0;
	dir = opendir(dirname);

	if(!dir)
	{
		fprintf(stderr, "No %s\n", dirname);
		return NULL;
	}
	
	fprintf(stderr, "Reading DIR %s\n", dirname);
	
	while((de = readdir(dir)))
	{
		fprintf(stderr, ">>%s\n", de->d_name);

		char *p = strchr(de->d_name, '.');
		if(!p || !strlen(ext))
			p = "";
		
		if(de->d_name[0] != '.')
		{
			struct stat ss;
			ss.st_mode = 0;
			ss.st_size = 0;
			sprintf(tmp, "%s%c%s", dirname, SEPARATOR, de->d_name);

			if(stat(tmp, &ss) == 0)
			{
				if((ss.st_mode & S_IFDIR) || (strcmp(p, ext) == 0))
				{
					flist[i].name = (char *)malloc(strlen(de->d_name) + 1);
					strcpy(flist[i].name, de->d_name);
					if(ss.st_mode & S_IFDIR)
						flist[i].size = -1;
					else
						flist[i].size = ss.st_size;
					i++;
				}
			}
		}
	}
	
	closedir(dir);

	if(count)
		*count = i;

	flist[i].name = NULL;
	i++;
	
	FileInfo *finfo = (FileInfo *)malloc(i * sizeof(FileInfo));
	memcpy(finfo, flist, i * sizeof(FileInfo));
	return finfo;
}

// Parent dir where files are looked for
char basedir[128];
char *BASEDIR(char *dir)
{
	static char *endp = NULL;
	if(!endp)
		endp = &basedir[strlen(basedir)];
		
	if(*dir == SEPARATOR)
		return dir;

	strcpy(endp, dir);
	return basedir;
}
