/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef FILES_H
#define FILES_H

#include <vector>
#include <string>

#ifdef _WIN32
#define SEPARATOR '\\'
#else
#define SEPARATOR '/'
#endif


class IFileRef
{
public:
	virtual int Read(char *dest, int offset, int length) = 0;
	virtual const char *GetFileName() = 0;
};

class FileData
{
public:
	FileData(std::string n = "", int s = 0) : name(n), size(s) {}

	std::string name;
	std::string path;
	int size;
	void *data;
	//...flags, date etc
};

class IFileList
{
public:

	IFileList() : marked(0), start(0) {};

	virtual ~IFileList() {}

	virtual int Size() = 0;

	virtual const char *GetName(int index) = 0;
	virtual int GetSize(int index) = 0;
	virtual IFileRef *GetFileRef(int index) = 0;

	virtual int Enter(int index) = 0;
	virtual char *GetCurDir() = 0;
	virtual void SetFilterFunc(int (*fcb)(const char *)) = 0;

	virtual int Flagged(int x) { return 0; }

	virtual int Changed() { return 0; }

	int Marked() { return marked; }
	void SetMarked(int m) { marked = m; }
	int Start() { return start; }
	void SetStart(int m) { start = m; }

protected:

	int marked;
	int start;

};

#endif
