/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef MUSIC_PLAYER_H
#define MUSIC_PLAYER_H

#include <vector>
#include "plugin.h"

class BlockFifo;

extern struct sound_plugin *current_plugin;

class MusicPlayer
{
public:
	MusicPlayer();

	//bool PlaySong(const char *fname);
	bool PlaySong(const char *name, char *data = NULL, int size = -1);

	int Pause();

	int GetMaxTunes() { return (current_plugin ? current_plugin->subtunes : -1); }
	int GetTune() {  return subtune; }

	void SetTune(int n);
	void SetVolume(int v);
	int GetVolume() { return vol; }

	int GetCSeconds() { return current_plugin ? total_samples / (current_plugin->freq*4/10) : 0; }

	const char *GetName() {  return (current_plugin ? current_plugin->name : ""); }
	const char *GetArtist() {  return (current_plugin ? current_plugin->artist : ""); }
	const char *GetCopyright() {  return (current_plugin ? current_plugin->copyright : ""); }
	const char *GetFormat() {  return (current_plugin ? current_plugin->format : ""); }

	int GetLength() { return (current_plugin ? current_plugin->length / 1000 : -1); }

	bool Playing() { return playing; }

	int CanHandle(const char *name);

	int GetNextSamples(short *dest, int len);

	void Stop() { songend_samples = 1; }

	void SetBlend(int b)
	{ 
		if(b < 0)
			b = 0;
		if(b > 255)
			b = 255;
		blend = b;
	}

	int GetBlend() { return blend; }

	int GetHeavy() { return heavy; }

protected:

	static void audio_cb(void *userdata, Uint8 *stream, int len);
	static void fill_audio(BlockFifo *af, int max);


	std::vector<sound_plugin*> sound_plugins;
	
	//struct sound_plugin *current_plugin;

	bool playing;

	int subtune;
	//int maxtune;
	//int last_tune;
	int vol;
	int total_samples;
	int songend_samples;
	//int lastvol;
	BlockFifo *audioFifo;

	int blend;
	int heavy;

	char temp_name[64];

	int InitSound(int freq);

};

#endif
