/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef ZIPFILELIST_H
#define ZIPFILELIST_H

#include "Files.h"
#include "zipint.h"

//struct zip;

class ZipDir;

class ZipFileList : public IFileList
{
public:

	ZipFileList(char *source);
	virtual ~ZipFileList();

	//virtual int Read(int index, char *dest, int offset, int length);
	virtual const char *GetFileName(int index);
	//virtual std::vector<FileRef> &GetNames();
	virtual int Enter(int index);

	virtual void SetFilterFunc(int (*fcb)(const char *))
	{
		printf("Setting %p %p\n", filter_cb, &filter_cb);
		filter_cb = fcb;
	}

	virtual const char *GetName(int index)
	{
		return filerefs[index].name.c_str();
	}

	virtual int GetSize(int index)
	{
		return filerefs[index].size;
	}

	int Size() { return filerefs.size(); }



	char *GetCurDir();

	//void *GetZipReference(int n);
	IFileRef *GetFileRef(int index);

protected:

	class ZipFileRef : public IFileRef
	{
	public:
		ZipFileRef(FileData &d)
		{
			data = d;
		}

		~ZipFileRef()
		{
			if(data.data)
			{
				struct zip *z = (struct zip *)data.data;
				free(z->cdir->entry->filename);
				free(z->cdir->entry);
				free(z->cdir);
				free(z->zn);
				free(z);
			}
		}

		const char *GetFileName() { return NULL; }

		int Read(char *dest, int offset, int length)
		{
			struct zip *z = (struct zip *)data.data;
			z->zp = fopen(z->zn, "rb");
			zip_file *zfp = zip_fopen_index(z, 0, 0);
			while(offset--)
				zip_fread(zfp, dest, 1);
			int l = zip_fread(zfp, dest, length);
			zip_fclose(zfp);
			fclose(z->zp);
			return length;
		}
	//protected:
		FileData data;
	};

	int (*filter_cb)(const char *);

	std::vector<FileData> filerefs;

	bool dirty;
	ZipDir *curdir;

	struct zip *zipf;
	int count;
	int dircount;

	char zipname[128];


	void Enter(char *dirname);

	void *GetZipReference(int index);

};

#endif
