/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "FileList.h"

#include <vector>

using namespace std;

//#ifdef _WIN32
const char *BaseName(const char *fname);
//#endif

#ifdef _WIN32
#define STRDUP _strdup
#include <io.h>


struct dirent
{
	char *d_name;
};

typedef struct
{
	struct dirent de;
	struct _finddata_t fd;
	intptr_t handle;
} DIR;


DIR *opendir(char *name)
{
	char tmp[256];
	DIR *dir = (DIR*)malloc(sizeof(DIR));
	dir->de.d_name = NULL;
	sprintf(tmp, "%s\\*", name);
	//printf("Looking for %s\n", tmp);
	dir->handle = _findfirst(tmp, &dir->fd);
	return dir;
}

struct dirent *readdir(DIR *dir)
{
	intptr_t rc = dir->handle;
	if(dir->de.d_name)
		rc = _findnext(dir->handle, &dir->fd);
	if(rc == -1)
		return NULL;
	dir->de.d_name = dir->fd.name;
	return &dir->de;
}

void closedir(DIR *dir)
{
	_findclose(dir->handle);
	free(dir);
}
#else
#define STRDUP strdup
#include <dirent.h>
#endif

FileList::FileList(char *dirname, int (*fcb)(void *, const char *), void *data) : 
	filter_cb(fcb),
	cb_data(data)
{
	if(dirname)
	{
		strcpy(curdir, dirname);
		Enter(dirname);
	}
	else
	{
		*curdir = 0;
	}

}

void FileList::AddDirectory(char *dirname, bool recursive)
{
	char tmp[128];
	DIR *dir;
	struct dirent *de;
	int i = 0;
	dir = opendir(dirname);

	if(dir)
	{
		while((de = readdir(dir)))
		{
			//fprintf(stderr, ">>%s\n", de->d_name);		
			if(de->d_name[0] != '.')
			{
				struct stat ss;
				ss.st_mode = 0;
				sprintf(tmp, "%s%c%s", dirname, SEPARATOR, de->d_name);
				if(stat(tmp, &ss) == 0)
				{
					if(!filter_cb || (ss.st_mode & S_IFDIR) || filter_cb(cb_data, de->d_name))
					{
						if(recursive && (ss.st_mode & S_IFDIR))
							AddDirectory(tmp, recursive);
						else
						{
							FileData fr;
							fr.path = string(tmp);
							fr.name = string(BaseName(tmp));
							if(ss.st_mode & S_IFDIR)
								fr.size = -1;
							else
								fr.size = ss.st_size;
							fr.data = NULL;
							filerefs.push_back(fr);
						}
					}
				}
			}
		}
		closedir(dir);
	}
}


void FileList::Enter(char *dirname)
{

	//for(int i=0; i<filerefs.size(); i++)
	//{
	//	free(fnames[i]);
	filerefs.clear();
	AddDirectory(dirname);
}



int FileList::Enter(int index)
{
	char old[128] = "";
	if(index == -1)
	{
		char *end = strrchr(curdir, SEPARATOR);
		if(end)
		{
			strcpy(old, curdir);
			*end = 0;
			if(!strchr(curdir, SEPARATOR))
			{
				*end = SEPARATOR;
				end[1] = 0;
			}
		}
	}
	else
		strcpy(curdir, filerefs[index].path.c_str());

	Enter(curdir);

	if(strlen(old))
	{
		printf("Old was '%s'\n", old);
		for(unsigned int i=0; i<filerefs.size(); i++)
		{
			printf("'%s'\n", filerefs[i].path.c_str());
			if(strcmp(filerefs[i].path.c_str(), old) == 0)
			{
				marked = i;
				return i;
			}
		}
	}

	marked = 0;
	return 0;

}

FileList::~FileList()
{
	//for(int i=0; i<fnames.size(); i++)
	//	free(fnames[i]);
}

IFileRef *FileList::GetFileRef(int index)
{
	RealFileRef *rf = new RealFileRef(filerefs[index]);
	return rf;
}


/*int FileList::Read(int index, char *dest, int offset, int length)
{
	int rc = -1;
	FILE *fp = fopen(filerefs[index].path.c_str(), "rb");
	if(fp)
	{
		fseek(fp, offset, SEEK_SET);
		rc = fread(dest, 1, length, fp);
		fclose(fp);
	}
	return rc;
}*/

//vector<FileRef> &FileList::GetNames()
//{
/*	if(dirty)
	{
		filerefs.clear();
		for(int i=0; i<fnames.size(); i++)
		{
			struct stat ss;
			ss.st_mode = 0;
			ss.st_size = 0;
			if(stat(fnames[i], &ss) == 0)
			{
				//if((ss.st_mode & S_IFDIR) || (strcmp(p, ext) == 0))
				{
					FileRef fr;

					fr.name = strrchr(fnames[i], SEPARATOR);
					if(!fr.name)
						fr.name = fnames[i];
					else
						fr.name++;

					if(ss.st_mode & S_IFDIR)
						fr.size = -1;
					else
						fr.size = ss.st_size;

					filerefs.push_back(fr);
				}
			}
		}
		dirty = false;
	}
*/
//	return filerefs;

//}




