/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#ifndef PLAYLIST_H
#define PLAYLIST_H

#include "Files.h"
#include "FileList.h"

#include <vector>
#include <string>

class PlayList : public FileList
{
public:

	PlayList(int (*fcb)(void *, const char *) = NULL, void *data = NULL);
	virtual ~PlayList();

	virtual const char *GetName(int index)
	{
		return filerefs[index].name.c_str();
	}

	virtual int GetSize(int index)
	{
		return filerefs[index].size;
	}

	int Size() { return filerefs.size(); }


	void AddFile(const char *name, int size, IFileRef *fr);
	void RemoveFile(int n);
	void MoveFile(int from, int to);

	const char *GetFileName(int index);

	IFileRef *GetFileRef(int index);

	int Flagged(int x) { return Current() == x; }

	void SetCurrent(int n);
	int Current();
	int Changed() { return changed; }

	void Prev(bool wrap);
	void Next(bool wrap);


	void Shuffle();
	void SetShuffle(bool on) { shuffle_on = on; Shuffle(); }

	void Save(const char *name);
	void Load(const char *name);

	//std::vector<FileRef*> &GetNames();

protected:

	struct PLData
	{
		std::string name;
		int size;
		IFileRef *fr;
	};

	class PLFileRef : public IFileRef
	{
	public:
		PLFileRef(IFileRef *fr) : real_ref(fr) {};
		//operator IFileRef* () { return real_ref; }

		int Read(char *dest, int offset, int length) { return real_ref->Read(dest, offset, length); }
		const char *GetFileName() { return real_ref->GetFileName(); }

	protected:
		IFileRef *real_ref;
	};

	std::vector<PLData> filerefs;
	int current;
	int shuffle_current;
	int changed;
	bool shuffle_on;
	bool shuffled;
	std::vector<int> shuffle_list;


};

#endif
