#include <stdio.h>
#include "M3uPlayList.h"
#include "Files.h"
#include "util.h"
            
// Actually supports pls and opl (OldPlayList) as well
M3uPlayList::M3uPlayList(const char* fname) 
{
    printf("M3uPlayList::M3uPlayList - Opening %s\n",fname);
    FILE* fp = fopen(fname,"r");
    if (!fp)
    {
        printf("M3uPlayList::M3uPlayList - Couldn't open file %s\n",fname);
        return;
    }
    char line[1024];
    char *dirend = strrchr(fname,SEPARATOR);
    int dirlen = strlen(fname)-strlen(dirend)+1;
    if (is_ext(fname,".m3u")) 
    {
        if(!(fgets(line,1023,fp)))
            return;
        if (line[0] != '#') // Simple
        {
            rewind(fp);
            printf("M3uPlayList::M3uPlayList - Parsed as simple m3u\n");
            for (;fgets(line,1023,fp);)
            {
                line[strlen(line)-1] = '\0';
                char path[1024];
                if (*line == SEPARATOR)
                    strcpy(path,line);
                else
                {
                    strncpy(path,fname,dirlen);
                    strcpy(&path[dirlen],line);
                }
                //printf("M3uPlayList::M3uPlayList - path = %s, name = %s\n",path,strrchr(path,SEPARATOR)+1);
                paths.push_back(string(path));
                names.push_back(string(strrchr(path,SEPARATOR)+1));
            }
        }
        else
        {
            printf("M3uPlayList::M3uPlayList - Parsed as extended m3u\n");
            for (;fgets(line,1023,fp);)
            {
                line[strlen(line)-1] = '\0';
                printf("%s\n",line);
                if (*line == '#')
                {
                    char *name = strchr(line,',');
                    if (!name || strlen(name) < 2)
                        continue;
                    printf("M3uPlayList::M3uPlayList - name = %s\n",name+1);
                    names.push_back(string(name+1));
                }
                else
                {
                    char path[1024];
                    if (*line == SEPARATOR)
                        strcpy(path,line);
                    else
                    {
                        strncpy(path,fname,dirlen);
                        strcpy(&path[dirlen],line);
                    }
                    printf("M3uPlayList::M3uPlayList - path = %s\n",path);
                    paths.push_back(string(path));
                }
            }
        }
    }
    else if (is_ext(fname,".pls"))
    {
        printf("M3uPlayList::M3uPlayList - Parsed as pls\n");
        for (;fgets(line,1023,fp);)
        {
            line[strlen(line)-1] = '\0';
            char* line2 = strrchr(line,'=');
            if (!line2 || strlen(line2) < 2)
                continue;
            if(line[0] == 'F')
            {
                char path[1024];
                if (*(line2+1) == SEPARATOR)
                    strcpy(path,&line2[1]);
                else
                {
                    strncpy(path,fname,dirlen);
                    strcpy(&path[dirlen],&line2[1]);
                }
                paths.push_back(string(path));
            }
            else if (line[0] == 'T')
            {
                char* name = (char*)malloc(strlen(line2+1)+1);
                strcpy(name,&line2[1]);
                names.push_back(name);
            }
        }
    }
    else if (is_ext(fname,".opl"))
    {
        printf("M3uPlayList::M3uPlayList - Parsed as opl (OldPlayList)\n");
        for (;fgets(line,1023,fp);)
        {
            line[strlen(line)-1] = '\0';
            char* line2 = strrchr(line,'=');
            if (!line2 || strlen(line2) < 2)
                continue;
            if(line[0] == 'F')
            {
                char path[1024];
                if (*(line2+1) == SEPARATOR)
                    strcpy(path,&line2[1]);
                else
                {
                    strncpy(path,fname,dirlen);
                    strcpy(&path[dirlen],&line2[1]);
                }
                paths.push_back(string(path));
            }
            else if (line[0] == 'T')
            {
                char* name = (char*)malloc(strlen(line2+1)+1);
                strcpy(name,&line2[1]);
                names.push_back(name);
            }
            else if (line[0] == 'I')
                index.push_back(atoi(line2+1));
        }
    }
    if (paths.size() != names.size())
    {
        printf("M3uPlayList::M3uPlayList - Title/Path mismatch, ignoring playlist (%d:%d)\n",paths.size(),names.size());
        paths.clear();
        names.clear();
    }
    if (is_ext(fname,".opl") && paths.size() != index.size())
    {
        printf("M3uPlayList::M3uPlayList - Index/Path mismatch, ignoring playlist (%d:%d)\n",paths.size(),names.size());
        paths.clear();
        names.clear();
        index.clear();
    }
    printf("M3uPlayList::M3uPlayList - Done, closing\n");
    fclose(fp);
}

M3uPlayList::~M3uPlayList()
{
    paths.clear();
    names.clear();
    index.clear();
}

void M3uPlayList::Next()
{
    if (!paths.size())
        return;
    paths.erase(paths.begin());
    names.erase(names.begin());
    if (index.size())
        index.erase(index.begin());
}

