/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <SDL.h>
#include "SDLScreen.h"
#include "Input.h"
#include "PlayerApp.h"
#include "State.h"

PlayerApp *app;

static int bcount = 0;
extern "C"
{
	void busy(int x)
	{
		if(x)
			bcount = (bcount+1) % 6;
		else
			bcount = 0;
		app->lcd->SetLed(LedControl::BUSY, bcount);
		app->lcd->Render();
		//SDL_Delay(1);
	}
}

extern BrowserState g_browserState;
extern PlayListState g_playlistState;
extern SaveState g_saveState;

#ifdef GP2X
#include <sys/times.h>
void hw_set_screen(bool on);
void hw_init();
void hw_set_cpu(int speed);
int hw_get_voltage();
#else
void hw_set_screen(bool on) {}
#endif

int current_clockfreq = -1;

int main(int argc, char **argv)
{
#ifdef GP2X
	struct tms buf;
	clock_t startt = times(&buf);
	fprintf(stderr, "main - Start %d\n", startt);
	hw_init();
#endif

	app = new PlayerApp(argc > 1 ? argv[1] : NULL);
	app->Layout();

	fprintf(stderr, "main - First Layout\n");

	g_screen.Update();

	Input *input = new Input();

	IState *currentState = &g_browserState;
	currentState->Enter();

	int display_on = 1;
	int input_hold = 0;
	int volt = -1;
	int count = 0;

	bool go_on = true;

	app->lcd->SetLed(LedControl::VOLT, 12);

	fprintf(stderr, "main - Entering mainloop\n");

#ifdef GP2X
	if(app->cpu_scaling)
	{
        printf("main - Clockfreq to 100\n");
		current_clockfreq = 100;
		hw_set_cpu(100);
	}
#endif

	while(go_on)
	{
		SDL_Delay(60);

		app->Update();
		currentState->Update();

		if(display_on)
			app->Render();

        int key = input->GetKey(60);

#ifdef GP2X
		if(count++ == 50)
		{
			count = 0;
			int v = hw_get_voltage();
			volt = volt == -1 ? v : (volt * 15 + v)>>4;
            app->lcd->SetLed(LedControl::VOLT, (v-18)*2/3);
			if(current_clockfreq > 0)
				app->lcd->SetLed(LedControl::EQ1, (current_clockfreq)/50);
		}
#endif

		if(key == Input::HOLD)
		{
			input_hold ^= 1;
			app->lcd->SetLed(LedControl::HOLD, input_hold);
		}

		if(!input_hold && key)
		{
			if(go_on = app->HandleKey(key))
			{
				if(key == Input::SELECT)
				{
					if(currentState != &g_playlistState)
						currentState = &g_playlistState;
					else
						currentState = &g_browserState;
					currentState->Enter();
				}
				else if (key == Input::SAVE)
                {
                    if(currentState != &g_saveState)
                        currentState = &g_saveState;
                    else
                        currentState = &g_browserState;
                    currentState->Enter();
                }
                else
					currentState->HandleKey(key);
#ifdef GP2X
				if(key == Input::BLANK)
				{
					display_on ^= 1;
					hw_set_screen((bool)display_on);
				}
				else if(key == Input::SPEED)
				{
					if((current_clockfreq -= 25) < 50)
						current_clockfreq = 250;
                    printf("main - Setting clockfreq to %d\n",current_clockfreq);
					hw_set_cpu(current_clockfreq);
					app->lcd->SetLed(LedControl::EQ1, (current_clockfreq)/50);
				}
                else 
#endif
                if(key == Input::QUIT)
                {
                    SDL_CloseAudio();
                    SDL_Quit();
                    go_on = false;
                }
			}
		}
	}

#ifdef GP2X
	hw_set_cpu(200);
	if(!display_on)
		hw_set_screen(true);
#endif

	delete app;

	return 0;
}
